﻿using UnityEngine;
using System.Collections;

public class GameState : MonoBehaviour {
	
	public enum GSType {TitleScreen, FlyingRound, Loadout, GameOver, NewGame, None};
	static GSType current_state = GSType.None;
	static GSType next_state = GSType.TitleScreen;
	static GameState the_state_manager;
	
	public int current_round = 0;
	public float score = 0;
	
	public GameObject the_zeppelin;
	public GameObject mouse_tracker;
	protected GameObject gz;
	protected GameObject mt;
		
	// Use this for initialization
	void Start () {
		if(the_state_manager) {
			Debug.Log("State manager already exists!");
			Destroy(this);
		} else {
			the_state_manager = this;	
		}
	}
	
	void Awake() {
		DontDestroyOnLoad(transform.gameObject);
	}
	
	// Update is called once per frame
	void Update () {
		//Debug.Log("States:" + current_state + " " + next_state);
		
		// Player can always press escape to lock cursor...
		if(Input.GetKeyDown(KeyCode.Escape)) {
			Screen.lockCursor = false;
		}
		
		if(current_state != next_state) {
			CloseState(current_state);
			StartState(next_state);
		} else {
		
			// Process state events...
			switch(current_state) {
			case GSType.TitleScreen:
				bool keypressed1 = Input.GetKeyDown(KeyCode.A) || 
					Input.GetKeyDown(KeyCode.Space) ||
						Input.GetKeyDown(KeyCode.D) ||
						Input.GetKeyDown(KeyCode.LeftArrow) ||
						Input.GetKeyDown(KeyCode.RightArrow) ||
						Input.GetMouseButtonDown(0) ||
						Input.GetMouseButtonDown(1);
				if(keypressed1) {
					next_state = GSType.NewGame;
				}
				break;
			case GSType.GameOver:
				bool keypressed2 = Input.GetMouseButtonDown(0) || Input.GetMouseButtonDown(1);
				//Debug.Log(keypressed2);
				if(keypressed2) {
					next_state = GSType.NewGame;
				}
				break;
			case GSType.FlyingRound:
				if(Input.GetKeyDown(KeyCode.Space)) {
					if(Time.timeSinceLevelLoad > 10f) {
						next_state = GSType.Loadout;
					}
				}
				break;
			case GSType.Loadout:
				if(Time.timeSinceLevelLoad > 6f) {
					if(next_state == GSType.Loadout) {
					//Debug.Log(Time.timeSinceLevelLoad);
						next_state = GSType.FlyingRound;
					}
				}
				break;
			}
		}
		//Debug.Log("States:" + current_state + " " + next_state);
	}
	
	public static void ChangeState(GSType state) {
		next_state = state;
	}
	
	// Total hack, but it gets the job done
	protected void CloseState(GSType state) {
		switch(state) {
		case GSType.FlyingRound:
			Screen.lockCursor = false;
			break;
		case GSType.TitleScreen:
			Destroy(gz);
			Destroy(mt);
			break;
		case GSType.GameOver:
			// Destroy and rebuild the zeppelin so we can start a new game...
			Destroy(gz);
			Destroy(mt);
			break;
		}
	}
	
	protected void StartState(GSType state) {
		switch(state) {
		case GSType.FlyingRound:
			//Screen.lockCursor = true;
			Application.LoadLevel("Round");
			current_round += 1;
			break;
		case GSType.GameOver:
			Screen.lockCursor = false;
			Application.LoadLevelAdditive("GameOver");
			Destroy(gz);
			break;
		case GSType.NewGame:
			gz = Instantiate(the_zeppelin) as GameObject;
			mt = Instantiate(mouse_tracker) as GameObject;
			next_state = GSType.Loadout;
			current_round = 0;
			score = 0;
			break;
		case GSType.Loadout:
			Application.LoadLevel("Buying");
			Destroy(gz);
			gz = Instantiate(the_zeppelin) as GameObject;
			gz.transform.localEulerAngles = new Vector3(0, 90, 0);
			//gz.Repair();
			break;
		case GSType.TitleScreen:
			Screen.lockCursor = false;
			current_round = 0;
			//Application.LoadLevel("TitleScreen"); // We never need to load the actual title screen
			break;
		}
		current_state = state;
		Debug.Log("Now in Scene: " + current_state);
	}
	
	public GameObject GetMT() {
		return mt;
	}
	
	public static GameObject GetMouseTracker() {
		return the_state_manager.GetMT();
	}
	
	public static GameState.GSType CurrentState() {
		return current_state;
	}
	
	public static int CurrentRound() {
		return the_state_manager.current_round;
	}
	
	public static void AddToScore(float points) {
		the_state_manager.score += points;
	}
	
	public static float GetScore() {
		return the_state_manager.score;
	}
	
	public Vector3 ZeppelinPosition() {
		if(gz) {
			return gz.transform.position;
		}
		return Vector3.zero;		
	}
	public static Vector3 GetZeppelinPosition() {
		return the_state_manager.ZeppelinPosition();
	}
}
