﻿using UnityEngine;
using System.Collections;

public class Bullet : MonoBehaviour {
	
	protected float start_time;
	public float lifespan = 1f;
	
	public bool is_moving = true;
	public float initial_bullet_force = 10f;
	public Vector3 initial_velocity = Vector3.zero;
	
	//protected float minimum_speed = 0.5f;
	//protected float last_speed = 99f;
	
	public float attack_damage = 1.0f;
	
	protected bool despawn = false;
	
	// Use this for initialization
	void Start () {
		//Vector3 local_forward = Vector3.zero Vector3.forward
		if(is_moving) {
			rigidbody.AddRelativeForce(Vector3.forward * initial_bullet_force);
			rigidbody.AddForce(initial_velocity);
		}
		start_time = Time.timeSinceLevelLoad;
	}
	
	void Awake() {
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		//transform.Translate(Vector3.forward / 2, Space.Self);
		
		//DespawnBySpeed();
		DespawnByLifetime();
		Despawn();
	}
	
	public void SetForce(float force) {
		initial_bullet_force = force;
	}
	public void SetVelocity(Vector3 vel) {
		initial_velocity = vel;
	}
	
	/*
	void DespawnBySpeed() {
		// TODO: if enabled, when the bullet drops below minimum speed, it gets despawned
		if(rigidbody) {
			if(minimum_speed > 0f) {
				if((rigidbody.velocity.sqrMagnitude < minimum_speed) && (last_speed < minimum_speed)) {
					despawn = true;
				}
				last_speed = rigidbody.velocity.sqrMagnitude;
			}
		}
	}
	*/
	
	void DespawnByLifetime() {
		// When the bullet reaches the end of its lifespan, despawn
		if (Time.timeSinceLevelLoad > start_time + lifespan) { despawn = true; }		
	}
	
	void Despawn() {
		if(despawn) {
			Destroy (gameObject);
		}
	}
	
	void OnCollisionEnter(Collision collision) {
		foreach (ContactPoint contact in collision.contacts) {
            Debug.DrawRay(contact.point, contact.normal, Color.white);
        }
		Enemy enemy = collision.gameObject.GetComponent<Enemy>();
		Mount mount = collision.gameObject.GetComponent<Mount>();
		if(enemy) {
			float effectiveness = enemy.ApplyAttack(attack_damage);
			if(effectiveness > 0f) {
				despawn = true;
			}
		}
		if(mount) {
			float effectiveness = mount.ApplyAttack(attack_damage);
			if(effectiveness > 0f) {
				despawn = true;
			}
		}
	}
	
/*	public void OnDrawGizmos () {
		Gizmos.color = Color.green;
		if(gameObject.layer == LayerMask.NameToLayer("EnemyBullets")) {
			Gizmos.color = Color.red;
		}		
		Gizmos.DrawLine(rigidbody.position, rigidbody.position + (rigidbody.velocity * 0.05f));
	}
	*/
}
