﻿using UnityEngine;
using System.Collections;

public class Weapon : Device {
	
	public GameObject ammo;
	public GameObject current_target = null;
	
	public enum WeaponSetupList {Default, BasicMachinegun, AimingMachinegun, EnemyMachinegun}
	public WeaponSetupList weapon_settings = WeaponSetupList.Default;
	
	//public float max_angle = 0.0f;
	//public float min_angle = 0.0f;
	
	public float rotation_speed = 0.5f; // Used by AimSpin
	public float rotation_angle = 45f; // Used by AimSweep
	public float rotation_period = 3f; // Used by AimSweep
	public float lerp_speed = 8f; // Used by AimLerp & AimAnticipate
	public float lookahead_time = 0.05f; // Used by AimAnticipate
	public float bullet_force = 10.0f;
	public bool inherit_velocity = false;
	public bool aim_at_mouse = false;
	public bool fire_on_mouse = false;
	public float rotation_limit = 0.0f;
	public bool only_fire_in_range = false;
	public float firing_angle_limit = 0.75f;
	//public float rotation_max_limit = 0.0f;
	//public float rotation_min_limit = 0.0f;
	
	protected Vector3 targeting_gizmo_position = Vector3.zero;
	
	
	public float cooldown_time = 0.1f;
	protected float last_cooldown_draw = -99f;
	
	public delegate void Aiming();
	public delegate bool FiringPermission();
	public delegate void Firing();
	public delegate void Targeting();
	public delegate bool TargetSwitching();	
	
	protected Aiming aim;
	protected FiringPermission can_fire;
	protected Firing fire;
	protected Targeting target;
	protected TargetSwitching switch_target;
	
	// Use this for initialization
	void Start () {
		aim = AimAnticipate;
		can_fire = FiringPermissionTrue;
		fire = FireBullet;		
		target = TargetBruteForceFindEnemy;
		switch_target = SwitchWhenInvalid;
		if(weapon_settings != WeaponSetupList.Default) {
			switch(weapon_settings) {
			case WeaponSetupList.BasicMachinegun:
				SetupBasicMachinegun();
				break;
			case WeaponSetupList.AimingMachinegun:
				SetupAimingMachinegun();
				//inherit_velocity = true;
				break;
			case WeaponSetupList.EnemyMachinegun:
				can_fire = FiringPermissionTrue;
				fire = FireBullet;		
				switch_target = SwitchWhenInvalid;
				bullet_force = 30.0f;
				target = TargetBruteForceFindPlayerWeapon;
				aim = AimNaive;
				break;
			default:
				break;
			}
		}
		if(aim_at_mouse) {
			target = TargetMouse;
			aim = AimLerp;
		}
	}
	
	void SetupBasicMachinegun() {
		aim = AimSweep;
		can_fire = FiringPermissionTrue;
		fire = FireBullet;		
		target = TargetBruteForceFindEnemy;
		switch_target = SwitchWhenInvalid;
		rotation_angle = 65f; // Used by AimSweep
		rotation_period = 1f; // Used by AimSweep
		bullet_force = 30.0f;
	}
	
	void SetupAimingMachinegun() {
		aim = AimLerp;//Anticipate;
		can_fire = FiringPermissionTrue;
		fire = FireBullet;		
		target = TargetBruteForceFindEnemy;
		switch_target = SwitchWhenInvalid;
		lookahead_time = 0.05f; // Used by AimAnticipate
		bullet_force = 30.0f;
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		
		// Get target
		if(switch_target()) {
			target();
		}
		// using current_target...
			
		// rotate towards target
		aim();
		
		KeepInBounds();
		
		if(!only_fire_in_range || TargetIsInRange()) {
			if(can_fire()) {
				if(!fire_on_mouse || (Input.GetMouseButton(0))) {
					last_cooldown_draw = Time.fixedTime;
					// Fire bullet
					fire();	
				}
			}
		}
	}
	
	/*protected Vector3 Aim() {
		Vector3 target_position = current_target.transform.position;
		target_position.y = 0;
		return target_position;
	}*/
	
	protected void FireBullet() {
		// Spawn a bullet, passing it the current aiming vector
		GameObject bullet = Instantiate(ammo) as GameObject;
		bullet.transform.position = transform.position + transform.forward;
		var rotate = transform.rotation;
		rotate.x = 0;
		rotate.z = 0;
		bullet.transform.rotation = rotate;
		Bullet b = bullet.GetComponent<Bullet>();
		b.SetForce(bullet_force);
		if(inherit_velocity) {
			b.SetVelocity(transform.parent.GetComponent<AutonomousVehicle>().Velocity);			
		}
	}
	
	protected void FireBeam() {
		// Spawn a bullet, passing it the current aiming vector
		GameObject bullet = Instantiate(ammo) as GameObject;
		bullet.transform.position = transform.position;
		bullet.transform.rotation = transform.rotation * Quaternion.Euler(90f,0,0);
		bullet.transform.parent = transform;
		Bullet b = bullet.GetComponent<Bullet>();
		b.is_moving = false;
		b.SetForce(0f);
	}
	
	protected void AimFixed() { // Just stays at a fixed angle. Probably need to do something to pick that angle?
		//transform.rotation = transform.parent.rotation; 
	}
	
	protected void AimSpin() { // Spins
		transform.localRotation = transform.localRotation * Quaternion.Euler(0, rotation_speed, 0);		
	}
	
	protected void AimSweep() { // Swings back and forth over an arc
		// fix rotation angle; as a workaround for euler limits
		rotation_angle = (rotation_angle < 90f) ? rotation_angle : 90f;		
		transform.localRotation = Quaternion.Slerp(
			Quaternion.Euler(0, -rotation_angle, 0),
			Quaternion.Euler(0, rotation_angle, 0),
			Mathf.PingPong(Time.fixedTime / rotation_period, 1f));
		targeting_gizmo_position = transform.localPosition + new Vector3(0, 4f, 0);
	}
	
	protected void AimNaive() {
		if(!current_target) { return; }
		Vector3 target_position = current_target.transform.position;
		target_position.y = 0;
		transform.LookAt(target_position);
		targeting_gizmo_position = target_position;
	}
	
	protected void AimLerp() {
		/* For later: (http://forum.unity3d.com/threads/36377-transform-LookAt-or-Quaternion-LookRotation-on-1-axis-only)
		 * var newRotation = Quaternion.LookRotation(transform.position - target.position, Vector3.forward);
		 * newRotation.x = 0.0;
		 * newRotation.y = 0.0;
		 * transform.rotation = Quaternion.Slerp(transform.rotation, newRotation, Time.deltaTime * 8);
		 */
		if(!current_target) { return; }
		Vector3 target_position = current_target.transform.position;
		target_position.y = 0;
		//var newRotation = Quaternion.LookRotation(current_target.transform.position - transform.position, Vector3.forward);
		var newRotation = Quaternion.LookRotation(target_position - transform.position, Vector3.up);
		
		//newRotation.x = 0;
		//newRotation.z = 0;
		transform.rotation = Quaternion.Slerp(transform.rotation, newRotation, Mathf.Clamp01(Time.fixedDeltaTime * lerp_speed));
		targeting_gizmo_position = target_position;
	}

	protected void AimAnticipate() {
		if(!current_target) { return; }
		// Guess at where the target is going to be
		if(!current_target.rigidbody) { AimLerp(); return; }
		Vector3 target_position = current_target.GetComponent<AutonomousVehicle>().PredictFuturePosition(lookahead_time);
		//Vector3 target_position = current_target.rigidbody.position + (current_target.rigidbody. * lookahead_time);
		//Debug.Log(current_target.rigidbody.position);
		//Debug.Log(current_target.rigidbody.velocity);
		//Debug.Log(lookahead_time);
		target_position.y = 0;
		var newRotation = Quaternion.LookRotation(target_position - transform.position, Vector3.forward);
		newRotation.x = 0;
		newRotation.z = 0;
		transform.rotation = Quaternion.Slerp(transform.rotation, newRotation, Time.fixedDeltaTime * lerp_speed);
		targeting_gizmo_position = target_position;
	}
	
	protected void AimAnticipateDistance() { // Takes distance into accouont when looking ahead
		if(!current_target) { return; }
		// Guess at where the target is going to be
		if(!current_target.rigidbody) { AimLerp(); return; }
		Vector3 current_target_position = current_target.rigidbody.position;
		float anticipation_time = (current_target_position - transform.position).magnitude * (lookahead_time * 0.02f);
		Vector3 target_position = current_target.GetComponent<AutonomousVehicle>().PredictFuturePosition(anticipation_time);
		//Vector3 target_position = current_target.rigidbody.position + (current_target.rigidbody. * lookahead_time);
		//Debug.Log(current_target.rigidbody.position);
		//Debug.Log(current_target.rigidbody.velocity);
		//Debug.Log(lookahead_time);
		target_position.y = 0;
		var newRotation = Quaternion.LookRotation(target_position - transform.position, Vector3.forward);
		newRotation.x = 0;
		newRotation.z = 0;
		transform.rotation = Quaternion.Slerp(transform.rotation, newRotation, Time.fixedTime * lerp_speed);
		targeting_gizmo_position = target_position;
	}
	
	protected void TargetNearestEnemy() {
		// Find the nearest enemy, return that as the new target
		
	}
	
	protected void TargetMouse() {
		if(Input.GetMouseButton(0)) {
			//current_target = 
		
		}
		current_target = GameState.GetMouseTracker();
	}
	
	protected void TargetBruteForceFindEnemy() {
		//Just grab the next Enemy object the game hands us. 
		// Brute force, unpredictable but not random. 
		// Use better target selection for real code.
		current_target = GameObject.Find("Enemy");
	}
	
	protected void TargetBruteForceFindPlayerWeapon() {
		//Just grab the next Enemy object the game hands us. 
		// Brute force, unpredictable but not random. 
		// Use better target selection for real code.
		
		current_target = GameObject.Find("Mount");
		/*
		var target_list = GameObject.FindGameObjectsWithTag("Mount");
		
		if(target_list.Length > 0) {
			current_target = target_list.GetValue(Random.Range(0, target_list.Length - 1)) as GameObject;
			if(current_target) { 
				//if(current_target.GetComponent<Mount>().GetHitpoints() > 0) {
					TargetSelected(); 
				//}
			}
		}
		*/
	}

	
	protected bool SwitchWhenInvalid() { // Only switch when the current target is invalid (usually null)
		if(current_target) {
			return false;
		}
		return true;
	}
	
	protected bool FiringPermissionTrue() {
		return true;
	}
	
	protected bool FiringPermissionCooldown() {
		if(cooldown_time + last_cooldown_draw < Time.fixedTime) {
			return true;
		}
		return false;
	}
	
	public static float ClampAngle (float angle, float min, float max)
    {
        if (angle < -360F)
            angle += 360F;
        if (angle > 360F)
            angle -= 360F;
        return Mathf.Clamp (angle, min, max);
    }
	
	bool TargetIsInRange() {
		if(! current_target) { return false; }
		Vector3 to_target = (current_target.transform.position - transform.position).normalized;
		Vector3 forward = transform.forward.normalized;
		float mag = Vector3.Dot(to_target, forward);
		//Debug.Log(mag);
		float dot_product_limit = firing_angle_limit;
		if (mag > dot_product_limit) { return true; }
		return false;		
	}
	
	protected void KeepInBounds() {
		//if((rotation_max_limit != 0.0f) || (rotation_min_limit != 0.0f)){
		if(rotation_limit != 0f) {
			float rmax = rotation_limit;
			float rmin = 0f - rotation_limit;
			Quaternion rot = gameObject.transform.localRotation;
			float y = rot.eulerAngles.y;
			y = (y > 180f) ? y - 360f : y;
			float delta = 0f;
			delta = (y > rmax) ? rmax - y : (y < rmin)? rmin - y: 0f;
			Quaternion delta_rot = Quaternion.Euler(0, delta, 0);
			rot = rot * delta_rot;
			gameObject.transform.localRotation = rot;
		}
	}	
	
	public void OnDrawGizmos() {
		Gizmos.color = Color.yellow;
		if(weapon_settings == WeaponSetupList.EnemyMachinegun) { Gizmos.color = Color.red; }
		Gizmos.DrawWireSphere(targeting_gizmo_position, 1.5f);
		Gizmos.DrawRay(transform.position, transform.localRotation.eulerAngles);
	}
	
	protected virtual void TargetSelected() {
		var steer = transform.parent.GetComponent<SteerForPursuit>();
		var targ = current_target.GetComponent<DetectableObject>();
		Debug.Log(steer);
		Debug.Log(targ);
		steer.Quarry = targ;
	}
}
